@@ -89,6 +89,8 @@ gboolean gdm_emergency_server = FALSE;
 
 gboolean gdm_first_login = TRUE;
 
+int gdm_in_signal = 0;
+
 /* Configuration options */
 gchar *GdmUser = NULL;
 gchar *GdmGroup = NULL;
@@ -280,9 +282,20 @@ gdm_config_parse (void)
     GdmWilling = gnome_config_get_string (GDM_KEY_WILLING);    
 
     GdmStandardXServer = gnome_config_get_string (GDM_KEY_STANDARD_XSERVER);    
-    if (ve_string_empty (GdmStandardXServer)) {
-	    g_free (GdmStandardXServer);
-	    GdmStandardXServer = g_strdup ("/usr/bin/X11/X");
+    if (ve_string_empty (GdmStandardXServer) ||
+	access (GdmStandardXServer, X_OK) != 0) {
+	    gdm_info (_("%s: Standard X server not found, trying alternatives"),
+			"gdm_config_parse");
+	    if (access ("/usr/X11R6/bin/X", X_OK) == 0) {
+		    g_free (GdmStandardXServer);
+		    GdmStandardXServer = g_strdup ("/usr/X11R6/bin/X");
+	    } else if (access ("/opt/X11R6/bin/X", X_OK) == 0) {
+		    g_free (GdmStandardXServer);
+		    GdmStandardXServer = g_strdup ("/opt/X11R6/bin/X");
+	    } else if (ve_string_empty (GdmStandardXServer)) {
+		    g_free (GdmStandardXServer);
+		    GdmStandardXServer = g_strdup ("/usr/bin/X11/X");
+	    }
     }
     GdmFlexibleXServers = gnome_config_get_int (GDM_KEY_FLEXIBLE_XSERVERS);    
     GdmXnest = gnome_config_get_string (GDM_KEY_XNEST);    
@@ -347,7 +360,7 @@ gdm_config_parse (void)
 
     if (ve_string_empty (GdmServAuthDir)) {
 	    gdm_text_message_dialog
-		    (gdm_cons_i18n (N_("No daemon/ServAuthDir specified in the configuration file")));
+		    (_("No daemon/ServAuthDir specified in the configuration file"));
 	    gdm_fail (_("%s: No authdir specified."), "gdm_config_parse");
     }
 
@@ -455,14 +468,18 @@ gdm_config_parse (void)
 		    server = GdmStandardXServer;
 	    } else if (access ("/usr/bin/X11/X", X_OK) == 0) {
 		    server = "/usr/bin/X11/X";
+	    } else if (access ("/usr/X11R6/bin/X", X_OK) == 0) {
+		    server = "/usr/X11R6/bin/X";
+	    } else if (access ("/opt/X11R6/bin/X", X_OK) == 0) {
+		    server = "/opt/X11R6/bin/X";
 	    }
 	    /* yay, we can add a backup emergency server */
 	    if (server != NULL) {
 		    int num = gdm_get_free_display (0 /* start */,
 						    0 /* server uid */);
-		    gdm_error (_("%s: XDMCP disabled and no local servers defined. Adding /usr/bin/X11/X on :%d to allow configuration!"),
+		    gdm_error (_("%s: XDMCP disabled and no local servers defined. Adding %s on :%d to allow configuration!"),
 			       "gdm_config_parse",
-			       num);
+			       server, num);
 
 		    gdm_emergency_server = TRUE;
 		    displays = g_slist_append
@@ -475,11 +492,11 @@ gdm_config_parse (void)
 		    GdmTimedLogin = NULL;
 	    } else {
 		    char *s = g_strdup_printf
-			    (gdm_cons_i18n (N_("XDMCP is disabled and gdm "
-					       "cannot find any local server "
-					       "to start.  Aborting!  Please "
-					       "correct the configuration %s "
-					       "and restart gdm.")),
+			    (_("XDMCP is disabled and gdm "
+			       "cannot find any local server "
+			       "to start.  Aborting!  Please "
+			       "correct the configuration %s "
+			       "and restart gdm."),
 			     GDM_CONFIG_FILE);
 		    gdm_text_message_dialog (s);
 		    gdm_fail (_("%s: XDMCP disabled and no local servers defined. Aborting!"), "gdm_config_parse");
@@ -498,10 +515,10 @@ gdm_config_parse (void)
 
     if (pwent == NULL) {
 	    char *s = g_strdup_printf
-		    (gdm_cons_i18n (N_("The gdm user does not exist. "
-					"Please correct gdm configuration %s "
-					"and restart gdm.")),
-				    GDM_CONFIG_FILE);
+		    (_("The gdm user does not exist. "
+		       "Please correct gdm configuration %s "
+		       "and restart gdm."),
+		     GDM_CONFIG_FILE);
 	    gdm_text_message_dialog (s);
 	    gdm_fail (_("%s: Can't find the gdm user (%s). Aborting!"), "gdm_config_parse", GdmUser);
     } else {
@@ -510,11 +527,11 @@ gdm_config_parse (void)
 
     if (GdmUserId == 0) {
 	    char *s = g_strdup_printf
-		    (gdm_cons_i18n (N_("The gdm user is set to be root, but "
-				       "this is not allowed since it can "
-				       "pose a security risk.  Please "
-				       "correct gdm configuration %s and "
-				       "restart gdm.")), GDM_CONFIG_FILE);
+		    (_("The gdm user is set to be root, but "
+		       "this is not allowed since it can "
+		       "pose a security risk.  Please "
+		       "correct gdm configuration %s and "
+		       "restart gdm."), GDM_CONFIG_FILE);
 	    gdm_text_message_dialog (s);
 	    gdm_fail (_("%s: The gdm user should not be root. Aborting!"), "gdm_config_parse");
     }
@@ -530,9 +547,9 @@ gdm_config_parse (void)
 
     if (grent == NULL) {
 	    char *s = g_strdup_printf
-		    (gdm_cons_i18n (N_("The gdm group does not exist. "
-				       "Please correct gdm configuration %s "
-				       "and restart gdm.")),
+		    (_("The gdm group does not exist. "
+		       "Please correct gdm configuration %s "
+		       "and restart gdm."),
 		     GDM_CONFIG_FILE);
 	    gdm_text_message_dialog (s);
 	    gdm_fail (_("%s: Can't find the gdm group (%s). Aborting!"), "gdm_config_parse", GdmGroup);
@@ -542,11 +559,11 @@ gdm_config_parse (void)
 
     if (GdmGroupId == 0) {
 	    char *s = g_strdup_printf
-		    (gdm_cons_i18n (N_("The gdm group is set to be root, but "
-				       "this is not allowed since it can "
-				       "pose a security risk. Please "
-				       "correct gdm configuration %s and "
-				       "restart gdm.")), GDM_CONFIG_FILE);
+		    (_("The gdm group is set to be root, but "
+		       "this is not allowed since it can "
+		       "pose a security risk. Please "
+		       "correct gdm configuration %s and "
+		       "restart gdm."), GDM_CONFIG_FILE);
 	    gdm_text_message_dialog (s);
 	    gdm_fail (_("%s: The gdm group should not be root. Aborting!"), "gdm_config_parse");
     }
@@ -585,11 +602,11 @@ gdm_config_parse (void)
     /* Enter paranoia mode */
     if (stat (GdmServAuthDir, &statbuf) == -1)  {
 	    char *s = g_strdup_printf
-		    (gdm_cons_i18n (N_("Server Authorization directory "
-				       "(daemon/ServAuthDir) is set to %s "
-				       "but this does not exist. Please "
-				       "correct gdm configuration %s and "
-				       "restart gdm.")), GdmServAuthDir,
+		    (_("Server Authorization directory "
+		       "(daemon/ServAuthDir) is set to %s "
+		       "but this does not exist. Please "
+		       "correct gdm configuration %s and "
+		       "restart gdm."), GdmServAuthDir,
 		     GDM_CONFIG_FILE);
 	    gdm_text_message_dialog (s);
 	    gdm_fail (_("%s: Authdir %s does not exist. Aborting."), "gdm_config_parse", GdmServAuthDir);
@@ -597,11 +614,11 @@ gdm_config_parse (void)
 
     if (! S_ISDIR (statbuf.st_mode)) {
 	    char *s = g_strdup_printf
-		    (gdm_cons_i18n (N_("Server Authorization directory "
-				       "(daemon/ServAuthDir) is set to %s "
-				       "but this is not a directory. Please "
-				       "correct gdm configuration %s and "
-				       "restart gdm.")), GdmServAuthDir,
+		    (_("Server Authorization directory "
+		       "(daemon/ServAuthDir) is set to %s "
+		       "but this is not a directory. Please "
+		       "correct gdm configuration %s and "
+		       "restart gdm."), GdmServAuthDir,
 		     GDM_CONFIG_FILE);
 	    gdm_text_message_dialog (s);
 	    gdm_fail (_("%s: Authdir %s is not a directory. Aborting."), "gdm_config_parse", GdmServAuthDir);
@@ -609,12 +626,12 @@ gdm_config_parse (void)
 
     if (statbuf.st_uid != GdmUserId || statbuf.st_gid != GdmGroupId)  {
 	    char *s = g_strdup_printf
-		    (gdm_cons_i18n (N_("Server Authorization directory "
-				       "(daemon/ServAuthDir) is set to %s "
-				       "but is not owned by user %s and group "
-				       "%s. Please correct the ownership or "
-				       "gdm configuration %s and restart "
-				       "gdm.")),
+		    (_("Server Authorization directory "
+		       "(daemon/ServAuthDir) is set to %s "
+		       "but is not owned by user %s and group "
+		       "%s. Please correct the ownership or "
+		       "gdm configuration %s and restart "
+		       "gdm."),
 		     GdmServAuthDir, GdmUser, GdmGroup,
 		     GDM_CONFIG_FILE);
 	    gdm_text_message_dialog (s);
@@ -624,13 +641,13 @@ gdm_config_parse (void)
 
     if (statbuf.st_mode != (S_IFDIR|S_IRWXU|S_IRGRP|S_IXGRP))  {
 	    char *s = g_strdup_printf
-		    (gdm_cons_i18n (N_("Server Authorization directory "
-				       "(daemon/ServAuthDir) is set to %s "
-				       "but has the wrong permissions, it "
-				       "should have permissions of 0750. "
-				       "Please correct the permissions or "
-				       "the gdm configuration %s and "
-				       "restart gdm.")),
+		    (_("Server Authorization directory "
+		       "(daemon/ServAuthDir) is set to %s "
+		       "but has the wrong permissions, it "
+		       "should have permissions of 0750. "
+		       "Please correct the permissions or "
+		       "the gdm configuration %s and "
+		       "restart gdm."),
 		     GdmServAuthDir, GDM_CONFIG_FILE);
 	    gdm_text_message_dialog (s);
 	    gdm_fail (_("%s: Authdir %s has wrong permissions %o. Should be 0750. Aborting."), "gdm_config_parse", 
@@ -688,10 +705,11 @@ gdm_daemonify (void)
     }
 
     if (pid < 0) 
-	gdm_fail (_("gdm_daemonify: fork() failed!"));
+	gdm_fail (_("%s: fork() failed!"), "gdm_daemonify");
 
     if (setsid() < 0)
-	gdm_fail (_("gdm_daemonify: setsid() failed: %s!"), strerror(errno));
+	gdm_fail (_("%s: setsid() failed: %s!"), "gdm_daemonify",
+		  strerror(errno));
 
     chdir (GdmServAuthDir);
     umask (022);
@@ -825,6 +843,8 @@ deal_with_x_crashes (GdmDisplay *d)
 		    char *xlog = g_strconcat (GdmLogDir, "/", d->name, ".log", NULL);
 		    int ii;
 
+		    closelog ();
+
 		    for (ii = 0; ii < sysconf (_SC_OPEN_MAX); ii++)
 			    close (ii);
 
@@ -888,11 +908,11 @@ deal_with_x_crashes (GdmDisplay *d)
 	     * the above script would have been defined and we'd run
 	     * it for them */
 	    const char *error =
-		    gdm_cons_i18n (N_("I cannot start the X server (your graphical "
-				      "interface).  It is likely that it is not set "
-				      "up correctly. You will need to log in on a "
-				      "console and rerun the X configuration "
-				      "program.  Then restart GDM."));
+		    _("I cannot start the X server (your graphical "
+		      "interface).  It is likely that it is not set "
+		      "up correctly. You will need to log in on a "
+		      "console and rerun the X configuration "
+		      "program.  Then restart GDM.");
 	    gdm_text_message_dialog (error);
     } /* else {
        * At this point .... screw the user, we don't know how to
@@ -1184,7 +1204,7 @@ gdm_cleanup_children (void)
 static void
 gdm_restart_now (void)
 {
-	gdm_info (_("Gdm restarting ..."));
+	gdm_info (_("GDM restarting ..."));
 	final_cleanup ();
 	if (stored_path != NULL)
 		gnome_setenv ("PATH", stored_path, TRUE);
@@ -1305,24 +1325,31 @@ main (int argc, char *argv[])
     sigset_t mask;
     struct sigaction term, child;
     FILE *pf;
+    poptContext ctx;
+    int nextopt;
 
     store_argv (argc, argv);
 
-    setlocale(LC_ALL, "");
-
     bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
-    bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
     textdomain (GETTEXT_PACKAGE);
 
+    setlocale (LC_ALL, "");
+
     /* Initialize runtime environment */
     umask (022);
 
-    gnome_program_init ("gdm", VERSION,
-			LIBGNOME_MODULE /* module_info */,
-			argc, argv,
-			GNOME_PARAM_POPT_TABLE, options,
-			GNOME_PARAM_CREATE_DIRECTORIES, FALSE,
-			NULL);
+    ctx = poptGetContext ("gdm", argc, (const char **) argv,
+			  options, 0);
+    while ((nextopt = poptGetNextOpt (ctx)) > 0 || nextopt == POPT_ERROR_BADOPT)
+	/* do nothing */ ;
+
+    if (nextopt != -1) {
+	    g_print (_("Error on option %s: %s.\nRun '%s --help' to see a full list of available command line options.\n"),
+		     poptBadOption (ctx, 0),
+		     poptStrerror (nextopt),
+		     argv[0]);
+	    exit (1);
+    }
 
     /* XDM compliant error message */
     if (getuid () != 0) {
@@ -1539,6 +1566,20 @@ gdm_signal_notify (gint8 signal)
 	signals_notified[index] |= 1 << shift;
 }
 
+static void
+send_slave_ack (GdmDisplay *d)
+{
+	if (d->master_notify_fd >= 0) {
+		char not[2];
+		not[0] = GDM_SLAVE_NOTIFY_ACK;
+		not[1] = '\n';
+		write (d->master_notify_fd, not, 2);
+	}
+	if (d->slavepid > 0) {
+		kill (d->slavepid, SIGUSR2);
+	}
+}
+
 static void
 gdm_handle_message (GdmConnection *conn, const char *msg, gpointer data)
 {
@@ -1575,7 +1616,7 @@ gdm_handle_message (GdmConnection *conn, const char *msg, gpointer data)
 			d->servpid = pid;
 			gdm_debug ("Got XPID == %ld", (long)pid);
 			/* send ack */
-			kill (slave_pid, SIGUSR2);
+			send_slave_ack (d);
 		}
 	} else if (strncmp (msg, GDM_SOP_SESSPID " ",
 		            strlen (GDM_SOP_SESSPID " ")) == 0) {
@@ -1593,7 +1634,7 @@ gdm_handle_message (GdmConnection *conn, const char *msg, gpointer data)
 			d->sesspid = pid;
 			gdm_debug ("Got SESSPID == %ld", (long)pid);
 			/* send ack */
-			kill (slave_pid, SIGUSR2);
+			send_slave_ack (d);
 		}
 	} else if (strncmp (msg, GDM_SOP_GREETPID " ",
 		            strlen (GDM_SOP_GREETPID " ")) == 0) {
@@ -1611,7 +1652,7 @@ gdm_handle_message (GdmConnection *conn, const char *msg, gpointer data)
 			d->greetpid = pid;
 			gdm_debug ("Got GREETPID == %ld", (long)pid);
 			/* send ack */
-			kill (slave_pid, SIGUSR2);
+			send_slave_ack (d);
 		}
 	} else if (strncmp (msg, GDM_SOP_CHOOSERPID " ",
 		            strlen (GDM_SOP_CHOOSERPID " ")) == 0) {
@@ -1629,7 +1670,7 @@ gdm_handle_message (GdmConnection *conn, const char *msg, gpointer data)
 			d->chooserpid = pid;
 			gdm_debug ("Got CHOOSERPID == %ld", (long)pid);
 			/* send ack */
-			kill (slave_pid, SIGUSR2);
+			send_slave_ack (d);
 		}
 	} else if (strncmp (msg, GDM_SOP_LOGGED_IN " ",
 		            strlen (GDM_SOP_LOGGED_IN " ")) == 0) {
@@ -1655,7 +1696,7 @@ gdm_handle_message (GdmConnection *conn, const char *msg, gpointer data)
 				gdm_safe_restart ();
 
 			/* send ack */
-			kill (slave_pid, SIGUSR2);
+			send_slave_ack (d);
 		}
 	} else if (strncmp (msg, GDM_SOP_DISP_NUM " ",
 		            strlen (GDM_SOP_DISP_NUM " ")) == 0) {
@@ -1676,7 +1717,7 @@ gdm_handle_message (GdmConnection *conn, const char *msg, gpointer data)
 			d->dispnum = disp_num;
 			gdm_debug ("Got DISP_NUM == %d", disp_num);
 			/* send ack */
-			kill (slave_pid, SIGUSR2);
+			send_slave_ack (d);
 		}
 	} else if (strncmp (msg, GDM_SOP_VT_NUM " ",
 		            strlen (GDM_SOP_VT_NUM " ")) == 0) {
@@ -1697,7 +1738,7 @@ gdm_handle_message (GdmConnection *conn, const char *msg, gpointer data)
 #endif
 			gdm_debug ("Got VT_NUM == %d", vt_num);
 			/* send ack */
-			kill (slave_pid, SIGUSR2);
+			send_slave_ack (d);
 		}
 	} else if (strncmp (msg, GDM_SOP_LOGIN " ",
 		            strlen (GDM_SOP_LOGIN " ")) == 0) {
@@ -1724,7 +1765,7 @@ gdm_handle_message (GdmConnection *conn, const char *msg, gpointer data)
 			d->login = g_strdup (p);
 			gdm_debug ("Got LOGIN == %s", p);
 			/* send ack */
-			kill (slave_pid, SIGUSR2);
+			send_slave_ack (d);
 		}
 	} else if (strncmp (msg, GDM_SOP_COOKIE " ",
 		            strlen (GDM_SOP_COOKIE " ")) == 0) {
@@ -1751,7 +1792,7 @@ gdm_handle_message (GdmConnection *conn, const char *msg, gpointer data)
 			d->cookie = g_strdup (p);
 			gdm_debug ("Got COOKIE == <secret>");
 			/* send ack */
-			kill (slave_pid, SIGUSR2);
+			send_slave_ack (d);
 		}
 	} else if (strncmp (msg, GDM_SOP_FLEXI_ERR " ",
 		            strlen (GDM_SOP_FLEXI_ERR " ")) == 0) {
@@ -1788,7 +1829,7 @@ gdm_handle_message (GdmConnection *conn, const char *msg, gpointer data)
 			
 			gdm_debug ("Got FLEXI_ERR == %d", err);
 			/* send ack */
-			kill (slave_pid, SIGUSR2);
+			send_slave_ack (d);
 		}
 	} else if (strncmp (msg, GDM_SOP_FLEXI_OK " ",
 		            strlen (GDM_SOP_FLEXI_OK " ")) == 0) {
@@ -1818,11 +1859,35 @@ gdm_handle_message (GdmConnection *conn, const char *msg, gpointer data)
 			
 			gdm_debug ("Got FLEXI_OK");
 			/* send ack */
-			kill (slave_pid, SIGUSR2);
+			send_slave_ack (d);
 		}
 	} else if (strcmp (msg, GDM_SOP_SOFT_RESTART) == 0) {
 		gdm_restart_mode = TRUE;
 		gdm_safe_restart ();
+	} else if (strncmp (msg, GDM_SOP_SYSLOG " ",
+		            strlen (GDM_SOP_SYSLOG " ")) == 0) {
+		char *p;
+		long pid;
+	       	int type;
+		p = strchr (msg, ' ');
+		if (p == NULL)
+			return;
+		p++;
+		if (sscanf (p, "%ld", &pid) != 1)
+			return;
+		p = strchr (p, ' ');
+		if (p == NULL)
+			return;
+		p++;
+		if (sscanf (p, "%d", &type) != 1)
+			return;
+
+		p = strchr (p, ' ');
+		if (p == NULL)
+			return;
+		p++;
+
+		syslog (type, "(child %ld) %s", pid, p);
 	} else if (strcmp (msg, GDM_SOP_START_NEXT_LOCAL) == 0) {
 		gdm_start_first_unborn_local (3 /* delay */);
 	} else if (strcmp (msg, GDM_SOP_HUP_ALL_GREETERS) == 0) {
@@ -1838,12 +1903,14 @@ gdm_handle_message (GdmConnection *conn, const char *msg, gpointer data)
 
 /* extract second word and the rest of the string */
 static void
-extract_dispname_xauthfile_cookie (const char *msg,
-				   char **dispname,
-				   char **xauthfile,
-				   char **cookie)
+extract_dispname_uid_xauthfile_cookie (const char *msg,
+				       char **dispname,
+				       uid_t *uid,
+				       char **xauthfile,
+				       char **cookie)
 {
 	const char *p;
+	int i;
 	char *pp;
 
 	*dispname = NULL;
@@ -1863,6 +1930,23 @@ extract_dispname_xauthfile_cookie (const char *msg,
 	if (pp != NULL)
 		*pp = '\0';
 
+	/* Get uid */
+	p = strchr (p, ' ');
+	if (p == NULL) {
+		*dispname = NULL;
+		g_free (*dispname);
+		return;
+	}
+	while (*p == ' ')
+		p++;
+
+	if (sscanf (p, "%d", &i) != 1) {
+		*dispname = NULL;
+		g_free (*dispname);
+		return;
+	}
+	*uid = i;
+
 	/* Get cookie */
 	p = strchr (p, ' ');
 	if (p == NULL) {
@@ -2014,7 +2098,8 @@ check_cookie (const char *file, const char *disp, const char *cookie)
 
 static void
 handle_flexi_server (GdmConnection *conn, int type, const char *server,
-		     const char *xnest_disp, const char *xnest_auth_file,
+		     const char *xnest_disp, uid_t xnest_uid,
+		     const char *xnest_auth_file,
 		     const char *xnest_cookie)
 {
 	GdmDisplay *display;
@@ -2027,6 +2112,8 @@ handle_flexi_server (GdmConnection *conn, int type, const char *server,
 		struct stat s;
 		gboolean authorized = TRUE;
 
+		seteuid (xnest_uid);
+
 		g_assert (xnest_auth_file != NULL);
 		g_assert (xnest_disp != NULL);
 		g_assert (xnest_cookie != NULL);
@@ -2045,6 +2132,11 @@ handle_flexi_server (GdmConnection *conn, int type, const char *server,
 			authorized = FALSE;
 		}
 
+		if (s.st_uid != xnest_uid)
+			authorized = FALSE;
+
+		seteuid (0);
+
 		if ( ! authorized) {
 			/* Sorry dude, you're not doing something
 			 * right */
@@ -2160,7 +2252,10 @@ notify_displays_int (const char *key, int val)
 		GdmDisplay *disp = li->data;
 		if (disp->master_notify_fd >= 0) {
 			gdm_fdprintf (disp->master_notify_fd,
-				      "%s %d\n", key, val);
+				      "%c%s %d\n",
+				      GDM_SLAVE_NOTIFY_KEY, key, val);
+			if (disp->slavepid > 0)
+				kill (disp->slavepid, SIGUSR2);
 		}
 	}
 }
@@ -2173,7 +2268,10 @@ notify_displays_string (const char *key, const char *val)
 		GdmDisplay *disp = li->data;
 		if (disp->master_notify_fd >= 0) {
 			gdm_fdprintf (disp->master_notify_fd,
-				      "%s %s\n", key, val);
+				      "%c%s %s\n",
+				      GDM_SLAVE_NOTIFY_KEY, key, val);
+			if (disp->slavepid > 0)
+				kill (disp->slavepid, SIGUSR2);
 		}
 	}
 }
@@ -2221,6 +2319,24 @@ update_config (const char *key)
 
 		notify_displays_int (GDM_NOTIFY_ALLOWREMOTEAUTOLOGIN, val);
 
+		goto update_config_ok;
+	} else if (is_key (key, GDM_KEY_SYSMENU)) {
+		gboolean val = gnome_config_get_bool (GDM_KEY_SYSMENU);
+		if (ve_bool_equal (val, GdmSystemMenu))
+			goto update_config_ok;
+		GdmSystemMenu = val;
+
+		notify_displays_int (GDM_NOTIFY_SYSMENU, val);
+
+		goto update_config_ok;
+	} else if (is_key (key, GDM_KEY_CONFIG_AVAILABLE)) {
+		gboolean val = gnome_config_get_bool (GDM_KEY_CONFIG_AVAILABLE);
+		if (ve_bool_equal (val, GdmConfigAvailable))
+			goto update_config_ok;
+		GdmConfigAvailable = val;
+
+		notify_displays_int (GDM_NOTIFY_CONFIG_AVAILABLE, val);
+
 		goto update_config_ok;
 	} else if (is_key (key, GDM_KEY_RETRYDELAY)) {
 		int val = gnome_config_get_int (GDM_KEY_RETRYDELAY);
@@ -2254,6 +2370,45 @@ update_config (const char *key)
 
 		notify_displays_string (GDM_NOTIFY_REMOTEGREETER, val);
 
+		goto update_config_ok;
+	} else if (is_key (key, GDM_KEY_TIMED_LOGIN) ||
+		   is_key (key, GDM_KEY_TIMED_LOGIN_ENABLE)) {
+		gboolean enable = gnome_config_get_bool (GDM_KEY_TIMED_LOGIN_ENABLE);
+		char *val;
+
+		/* if not enabled, we just don't care */
+		if ( ! enable && ! GdmTimedLoginEnable)
+			goto update_config_ok;
+
+		val = gnome_config_get_string (GDM_KEY_TIMED_LOGIN);
+		if (strcmp (ve_sure_string (val),
+			    ve_sure_string (GdmTimedLogin)) == 0 &&
+		    ve_bool_equal (enable, GdmTimedLoginEnable)) {
+			g_free (val);
+			goto update_config_ok;
+		}
+
+		GdmTimedLoginEnable = enable;
+		g_free (GdmTimedLogin);
+		if (GdmTimedLoginEnable) {
+			GdmTimedLogin = val;
+		} else {
+			g_free (val);
+			GdmTimedLogin = NULL;
+		}
+
+		notify_displays_string (GDM_NOTIFY_TIMED_LOGIN,
+					ve_sure_string (GdmTimedLogin));
+
+		goto update_config_ok;
+	} else if (is_key (key, GDM_KEY_TIMED_LOGIN_DELAY)) {
+		int val = gnome_config_get_int (GDM_KEY_TIMED_LOGIN_DELAY);
+		if (val == GdmTimedLoginDelay)
+			goto update_config_ok;
+		GdmTimedLoginDelay = val;
+
+		notify_displays_int (GDM_NOTIFY_TIMED_LOGIN_DELAY, val);
+
 		goto update_config_ok;
 	} else if (is_key (key, GDM_KEY_XDMCP)) {
 		gboolean val = gnome_config_get_bool (GDM_KEY_XDMCP);
@@ -2349,7 +2504,7 @@ gdm_handle_user_message (GdmConnection *conn, const char *msg, gpointer data)
 			return;
 		}
 		handle_flexi_server (conn, TYPE_FLEXI, GdmStandardXServer,
-				     NULL, NULL, NULL);
+				     NULL, 0, NULL, NULL);
 	} else if (strncmp (msg, GDM_SUP_FLEXI_XSERVER " ",
 		            strlen (GDM_SUP_FLEXI_XSERVER " ")) == 0) {
 		char *name;
@@ -2391,12 +2546,14 @@ gdm_handle_user_message (GdmConnection *conn, const char *msg, gpointer data)
 		g_free (name);
 
 		handle_flexi_server (conn, TYPE_FLEXI, command,
-				     NULL, NULL, NULL);
+				     NULL, 0, NULL, NULL);
 	} else if (strncmp (msg, GDM_SUP_FLEXI_XNEST " ",
 		            strlen (GDM_SUP_FLEXI_XNEST " ")) == 0) {
 		char *dispname = NULL, *xauthfile = NULL, *cookie = NULL;
+		uid_t uid;
 
-		extract_dispname_xauthfile_cookie (msg, &dispname, &xauthfile, &cookie);
+		extract_dispname_uid_xauthfile_cookie (msg, &dispname, &uid,
+						       &xauthfile, &cookie);
 
 		if (dispname == NULL) {
 			/* Something bogus is going on, so just whack the
@@ -2417,7 +2574,7 @@ gdm_handle_user_message (GdmConnection *conn, const char *msg, gpointer data)
 		}
 		
 		handle_flexi_server (conn, TYPE_FLEXI_XNEST, GdmXnest,
-				     dispname, xauthfile, cookie);
+				     dispname, uid, xauthfile, cookie);
 
 		g_free (dispname);
 		g_free (xauthfile);
