@@ -25,8 +25,10 @@
 #include <sys/stat.h>
 #include <sys/wait.h>
 #include <sys/socket.h>
+#include <sys/ioctl.h>
 #include <fcntl.h>
 #include <netinet/in.h>
+#include <net/if.h>
 #include <arpa/inet.h>
 #include <netdb.h>
 #include <errno.h>
@@ -491,8 +493,7 @@ gdm_exec_wait (char * const *argv, gboolean no_display,
 		open ("/dev/null", O_RDWR); /* open stderr - fd 2 */
 
 		if (de_setuid) {
-			seteuid (getuid ());
-			setegid (getgid ());
+			gdm_desetuid ();
 		}
 
 		openlog ("gdm", LOG_PID, LOG_DAEMON);
@@ -714,7 +715,9 @@ gdm_peek_local_address_list (void)
 		g_free (buf);
 		if (sockfd != gdm_xdmcpfd)
 			close (sockfd);
-		return NULL;
+		addr = g_new0 (struct in_addr, 1);
+		addr->s_addr = INADDR_LOOPBACK;
+		return g_list_append (NULL, addr);
 	}
 
 	ifr = ifc.ifc_req;
@@ -750,16 +753,20 @@ gdm_peek_local_address_list (void)
 	/* host based fallback, will likely only get 127.0.0.1 i think */
 
 	if (gethostname (hostbuf, BUFSIZ-1) != 0) {
-		gdm_error (_("%s: Could not get server hostname: %s!"),
+		gdm_debug ("%s: Could not get server hostname: %s!",
 			   "gdm_peek_local_address_list",
 			   g_strerror (errno));
-		return NULL;
+		addr = g_new0 (struct in_addr, 1);
+		addr->s_addr = INADDR_LOOPBACK;
+		return g_list_append (NULL, addr);
 	}
 	he = gethostbyname (hostbuf);
 	if (he == NULL) {
-		gdm_error (_("%s: Could not get address from hostname!"),
+		gdm_debug ("%s: Could not get address from hostname!",
 			   "gdm_peek_local_address_list");
-		return NULL;
+		addr = g_new0 (struct in_addr, 1);
+		addr->s_addr = INADDR_LOOPBACK;
+		return g_list_append (NULL, addr);
 	}
 	for (i = 0; he->h_addr_list[i] != NULL; i++) {
 		struct in_addr *laddr = (struct in_addr *)he->h_addr_list[i];
@@ -814,6 +821,8 @@ gdm_is_loopback_addr (struct in_addr *ia)
 gboolean
 gdm_setup_gids (const char *login, gid_t gid)
 {
+	/* FIXME: perhaps for *BSD there should be setusercontext
+	 * stuff here */
 	if (setgid (gid) < 0)  {
 		gdm_error (_("Could not setgid %d. Aborting."), (int)gid);
 		return FALSE;
@@ -827,4 +836,23 @@ gdm_setup_gids (const char *login, gid_t gid)
 	return TRUE;
 }
 
+void
+gdm_desetuid (void)
+{
+	uid_t uid = getuid (); 
+	gid_t gid = getgid (); 
+
+#ifdef HAVE_SETRESUID
+	{
+		int setresuid(uid_t ruid, uid_t euid, uid_t suid);
+		int setresgid(gid_t rgid, gid_t egid, gid_t sgid);
+		setresuid (uid, uid, uid);
+		setresgid (gid, gid, gid);
+	}
+#else
+	seteuid (getuid ());
+	setegid (getgid ());
+#endif
+}
+
 /* EOF */
